#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            sourceNibeModbus.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.6e
#
# Date:            14.01.2015
#
# Description:     TaloLogger configuration wrapper for Nibe Modbus source.
#                  
# Version history: ** 14.01.2015 v1.6e (Olli Lammi) **
#                  Added to taloLogger. 
#
###########################################################################

# Imports

import string

from modules.core import log
from modules.core import configuration
from modules.core import dataSource

from modules.datasources.modbus import sourceModbus 


###########################################################################

# Classes

class NibeModbusConf(log.Logging, configuration.Configurable, dataSource.DataSource):
    def __init__(self, modname):
        log.Logging.__init__(self, 'NibeModbusConf')
        configuration.Configurable.__init__(self)
        self.setModuleName(modname)
        dataSource.DataSource.__init__(self)
        self.setID(modname)
        
        self.privateConf = None
        self.modbusSource = None
        self.portName = None
        
    def getServiceName(self):
        return self.getModuleName()

    def handleConfiguration(self, conf):
        self.portName = conf.getValue('PORTNAME', '', self.getModuleName())

        self.privateConf = configuration.Configuration()
        self.privateConf.setValue('TYPE', 'RTU', self.getModuleName())
        self.privateConf.setValue('PORTNAME', self.portName, self.getModuleName())
        self.privateConf.setValue('BAUDRATE', '9600', self.getModuleName())
        self.privateConf.setValue('PARITY', 'NONE', self.getModuleName())
        self.privateConf.setValue('STOPBITS', '1', self.getModuleName())
        self.privateConf.setValue('DATABITS', '8', self.getModuleName())
        self.privateConf.setValue('RESPONSETIMEOUT', '3.0', self.getModuleName())

        holdingregisters = []
        holdingregisters.append('Outdoor:1:40004:INT:1')
        
        self.privateConf.setValue('HOLDINGREGISTER', holdingregisters, self.getModuleName())

        return (1, '')
            
    def initConfiguration(self):        
        self.modbusSource = sourceModbus.ModbusConf(self.getModuleName())

        (stat, msg) = self.modbusSource.handleConfiguration(self.privateConf)
        if stat != 1:
            return (stat, msg)
        
        (stat, msg) = self.modbusSource.initConfiguration()
        if stat != 1:
            return (stat, msg)
                
        return (1, '')

    def releaseConfiguration(self):
        if self.modbusSource != None:
            self.modbusSource.releaseConfiguration()
            self.modbusSource = None
        return (1, '')

    def runDataSourceQueryCommandImpl(self, cmds):
        return self.modbusSource.runDataSourceQueryCommandImpl(cmds)

    @staticmethod
    def getModuleTypeName():
        return 'NIBEMODBUS'

    @staticmethod
    def getAllowedConfigurationKeys():
        return (['PORTNAME'], [])

