#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            raspberryGpioThread.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.6d
#
# Date:            28.12.2014
#
# Description:     Standalone python process to interact with GPIO.
#                  Should be executed with used id 0 (root).
#                  
# Requirements:    Python interpreter 2.6 or newer (www.python.org)
#
# Version history: ** 28.12.2014 v1.6d (Olli Lammi) **
#                  First version.
#
###########################################################################

# Commands:
#
#     hello
#     input <channel> [<flag>]   (<flag> = NONE|PULLUP|PULLDOWN)
#     init
#     read <channel>
#     exit

# Imports

import sys, os 
import string

import RPi.GPIO as GPIO

###########################################################################

# Functions

hasInit = False
channels = {}

def doOutput(str):
    sys.stdout.write(str + chr(10))
    sys.stdout.flush()

def startsWith(str, needle):
    if len(needle) <= 0:
        return True
    if len(str) < len(needle):
        return False
    if str[:len(needle)] == needle:
        return True
    return False

def handleCommand(buff):
    if buff == 'exit':
        return False

    if buff == 'hello':
        doOutput('OK')
        return True

    if buff == 'init':
        return handleInit()
    
    if startsWith(buff, 'input'):
        return handleInput(buff[5:])

    if startsWith(buff, 'read'):
        return handleRead(buff[4:])

    doOutput('ERROR:Invalid command')
    return True 

def handleInit():
    global hasInit
    if hasInit:
        doOutput('ERROR:Already init')
        return True

    for key in channels.keys():
        flag = string.strip(string.upper(channels[key]))
        if flag == 'NONE':
            GPIO.setup(int(key), GPIO.IN)
        elif flag == 'PULLUP':
            GPIO.setup(int(key), GPIO.IN, pull_up_down=GPIO.PUD_UP)
        elif flag == 'PULLDOWN':
            GPIO.setup(int(key), GPIO.IN, pull_up_down=GPIO.PUD_DOWN)
        else:
            doOutput('ERROR:Invalid flag')
            return False

    hasInit = True
    doOutput('OK')
    return True


def handleInput(cmd):
    if hasInit:
        doOutput('ERROR:Already init')
        return True
    temp = string.split(string.strip(cmd), ' ')
    if len(temp) < 1:
        doOutput('ERROR:Invalid input')
        return True

    flag = 'NONE'
    if len(temp) > 1:
        flag = string.upper(string.strip(temp[1]))
    channel = string.strip(temp[0])    
    channels[channel] = flag
    doOutput('OK')
    return True


def handleRead(cmd):
    if not hasInit:
        doOutput('ERROR:Not init')
        return True

    temp = string.split(string.strip(cmd), ' ')
    if len(temp) != 1:
        doOutput('ERROR:Invalid read')
        return True

    channel = string.strip(temp[0])    

    if not channels.has_key(channel):
        doOutput('ERROR:No channel configured')
        return True
    
    res = GPIO.input(int(channel))
    doOutput("%d" % res)
    return True


###########################################################################

# Main


buff = ""

GPIO.setmode(GPIO.BOARD)

while 1:
    tempc = sys.stdin.read(1)
    if not tempc:
        break
    if tempc == chr(10):
        if not handleCommand(buff):
            break
        buff = ""
    else:
        buff = buff + tempc

if hasInit:        
  GPIO.cleanup()
sys.exit(0)

