#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            sourceRPiGPIO.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.6d
#
# Date:            28.12.2014
#
# Description:     TaloLogger configuration wrapper for RPiGPIO source.
#                  
# Version history: ** 28.12.2014 v1.6d (Olli Lammi) **
#                  Added to taloLogger. 
#
###########################################################################

# Imports

import string

from modules.core import configuration
from modules.core import dataSource

import raspberryPiGPIO


###########################################################################

# Constants

INPUT_FLAGS = ['NONE', 'PULLUP', 'PULLDOWN']

###########################################################################

# Classes

class GPIOConf(raspberryPiGPIO.GPIOSource, configuration.Configurable, dataSource.DataSource):
    def __init__(self, modname):
        configuration.Configurable.__init__(self)
        self.setModuleName(modname)
        raspberryPiGPIO.GPIOSource.__init__(self)
        dataSource.DataSource.__init__(self)
        self.setID(modname)
        
    def getServiceName(self):
        return self.getModuleName()

    def handleConfiguration(self, conf):
        self.setDatapoints(self.handleGPIOConf(conf)) 
        return (1, '')

    def handleGPIOConf(self, conf):
        pointmap = {}
        for item in conf.getValue('INPUT', [], self.getModuleName()):
            temps = string.split(item, ':', 3)
            if len(temps) < 3:
                self.log("ERROR: Invalid INPUT configuration: " + item)
            else:
                try:
                    key = string.strip(temps[0])
                    channel = int(string.strip(temps[1]))
                    flag = string.upper(string.strip(temps[2]))
                except:
                    key = ''
                    
                if flag not in INPUT_FLAGS:
                    self.log("ERROR: Invalid INPUT flag.")
                    key = ''
                if len(key) <= 0:
                    self.log("ERROR: Invalid INPUT configuration: " + item)
                else:
                    pointmap[key] = [channel, flag]
        return pointmap
            
    def initConfiguration(self):
        if len(self.DATAPOINTS.keys()) <= 0:
            return (0, 'No data points configured for RPIGPIO datasource module.')

        if not self.initModule():        
            return (0, 'Unable to initialize GPIO thread.')
        
        return (1, '')

    def releaseConfiguration(self):
        self.closeModule()
        return (1, '')

    def runDataSourceQueryCommandImpl(self, cmds):
        return self.runQueryCommand(cmds)

    @staticmethod
    def getModuleTypeName():
        return 'RPIGPIO'

    @staticmethod
    def getAllowedConfigurationKeys():
        return ([], ['INPUT'])
