#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            sourceShell.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.4i
#
# Date:            13.02.2012
#
# Description:     TaloLogger configuration wrapper for Shell data source.
#                  
# Version history: ** 07.11.2011 v1.4g (Olli Lammi) **
#                  Added to taloLogger. 
#
#                  ** 13.02.2012 v1.4i (Olli Lammi) **
#                  Multisupport.
#
###########################################################################

# Imports

import os, string

from modules.core import configuration
from modules.core import dataSource

import shellSource


###########################################################################

# Classes

class ShellConf(shellSource.ShellSource, configuration.Configurable, dataSource.DataSource):
    def __init__(self, modname):
        configuration.Configurable.__init__(self)
        self.setModuleName(modname)
        shellSource.ShellSource.__init__(self, '', '', '')
        dataSource.DataSource.__init__(self)
        self.setID(modname)
        
    def getServiceName(self):
        return self.getModuleName()

    def handleConfiguration(self, conf): 
        self.SH_BINARY = conf.getValue('LOCATION', '', self.getModuleName())
        self.SH_PARAM = conf.getValue('PARAMETERS', '', self.getModuleName())
        self.SH_REGEXP = conf.getValue('REGEXP', '', self.getModuleName())
        return (1, '')
        
    def initConfiguration(self):
        self.setTimeout(self.dsTimeout)

        # check the existence of the executable
        if not os.access(self.SH_BINARY, os.X_OK):
            return (0, 'ShellSource: Unable to find or execute Shell executable: ' + self.SH_BINARY)
        return (1, '')

    def runDataSourceQueryCommandImpl(self, cmds):
        return self.runQueryCommand(cmds)

    @staticmethod
    def getModuleTypeName():
        return 'SHELL'

    @staticmethod
    def getAllowedConfigurationKeys():
        return (['LOCATION', 'PARAMETERS', 'REGEXP'], [])
